unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TPassState = (psBlank, psTesting, psPass, psFail);

const
  AM_POSTSTR = WM_USER + $99;

type
  TMainForm = class(TForm)
    CheckTime: TTimer;
    ResultMemo: TMemo;
    Panel1: TPanel;
    PassFailLabel: TLabel;
    TestOldButton: TButton;
    TestNewButton: TButton;
    DetectGroup: TGroupBox;
    Label1: TLabel;
    DetRefLabel: TLabel;
    Label2: TLabel;
    DetTestLabel: TLabel;
    BlurbPanel: TPanel;
    BlurbLabel: TLabel;
    procedure CheckTimeTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TestButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

  protected
    procedure SetMasterPassState(NewPassState: TPassState);
    procedure AMPostStr(var Msg: TMessage); message AM_POSTSTR;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, AIOUSB;

const
  ERROR_DEVICE_REMOVED = 1617;

type
  TDetState = (dsNone, dsRef, dsTest, dsBoth, dsPastless);

  TTestData = record
    ReadDI, ReadMasterMHz,
    WriteDI, WriteMasterMHz: LongWord;
  end;

  TIOThread = class(TThread)
    DI: LongWord;
    bFailed, bDone: Boolean;
    //hDoneMutex: THandle;
    function IsDone: Boolean;
    constructor Create; virtual;
    destructor Destroy; override;
  end;

  TWriteThread = class(TIOThread)
    procedure Execute; override;
  end;

  TReadThread = class(TIOThread)
    bEatFirstWord: Boolean;
    procedure Execute; override;
  end;

  TCheckFIFOWriteThread = class(TThread)
    DI: LongWord;
    bFailed: Boolean;
    FIFOPackets: Integer;
    procedure Execute; override;
  end;

var
  RefDI, TestDI: LongWord;
  bTestingDI, bTestingDO: Boolean;
  LastDetState: TDetState = dsPastless;
  LastPassState: TPassState = psBlank;
  TestScript: array[0..11] of TTestData = (
    (ReadMasterMHz: 01; WriteMasterMHz:  0),
    (ReadMasterMHz:  0; WriteMasterMHz: 01),
    (ReadMasterMHz: 01; WriteMasterMHz:  0),
    (ReadMasterMHz:  0; WriteMasterMHz: 01),
    (ReadMasterMHz: 07; WriteMasterMHz:  0),
    (ReadMasterMHz:  0; WriteMasterMHz: 07),
    (ReadMasterMHz: 07; WriteMasterMHz:  0),
    (ReadMasterMHz:  0; WriteMasterMHz: 07),
    (ReadMasterMHz: 32; WriteMasterMHz:  0),
    (ReadMasterMHz:  0; WriteMasterMHz: 32),
    (ReadMasterMHz: 32; WriteMasterMHz:  0),
    (ReadMasterMHz:  0; WriteMasterMHz: 32)
  );
  InBuf, OutBuf: array of Word;
  TicksPerSec: Int64;
  PostedStr: array of String;
  hPostedStrMutex: THandle;
  bDiscoRef, bDiscoTest: Boolean;

const
  RefID: LongWord = $6101D001;
  WriteWords = 4*1024;
  ReadWords = 1024;

procedure TMainForm.SetMasterPassState(NewPassState: TPassState);
begin
  if NewPassState = LastPassState then Exit;

  case NewPassState of
    psBlank: begin
      PassFailLabel.Caption := '';
      PassFailLabel.Color := clBtnFace;
      PassFailLabel.Font.Color := clBtnText;
    end;
    psTesting: begin
      PassFailLabel.Color := $FF0000;
      PassFailLabel.Font.Color := $FFFFFF;
      PassFailLabel.Caption := 'Testing...';
    end;
    psPass: begin
      PassFailLabel.Color := $00FF00;
      PassFailLabel.Font.Color := $000000;
      PassFailLabel.Caption := 'PASS';
    end;
    psFail: begin
      PassFailLabel.Color := $0000FF;
      PassFailLabel.Font.Color := $FFFFFF;
      PassFailLabel.Caption := 'FAIL';
    end;
  end;
  LastPassState := NewPassState;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  QueryPerformanceFrequency(TicksPerSec);

  hPostedStrMutex := CreateMutex(nil, False, nil);

  Application.Title := Caption;

  DetectGroup.Align := alTop;
  TestOldButton.Align := alTop;
  TestNewButton.Align := alTop;

  BlurbLabel.Caption := 'Plug in the reference board and a board to test, connect them with a 1:1 cable, and click one of the Test buttons.';

  with PassFailLabel.Font do Size := 2*Size;
  //Deal with anchor problems.
  BlurbLabel.SetBounds(8, 8, BlurbPanel.Width - 16, BlurbPanel.Height - 16);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  CloseHandle(hPostedStrMutex);
end;

procedure TMainForm.CheckTimeTimer(Sender: TObject);
var
  I, Mask, Status, PID: LongWord;
  ThisDetState: TDetState;
  bGotEm: Boolean;

begin
  if not CheckTime.Enabled then Exit;

  for I := 0 to 31 do AIOUSB_CloseDevice(I);
  Mask := GetDevices;
  RefDI := GetDeviceByEEPROMData(0, SizeOf(RefID), @RefID);

  TestDI := diNone;
  for I := 0 to 31 do
    if I <> RefDI then
      if (Mask and (1 shl I)) <> 0 then begin
        Status := QueryDeviceInfo(I, @PID, nil, nil, nil, nil);
        if Status <> ERROR_SUCCESS then Continue;
        if PID < $800C then Continue;
        if PID > $800F then Continue;

        if TestDI = diNone then begin
          bTestingDI := PID <> $800E;
          bTestingDO := PID <> $800D;
          TestDI := I;
        end
        else begin
          TestDI := diOnly;
        end;
      end
  ;

  if RefDI = diNone then begin
    //DetRefLabel.ParentColor := True;
    //DetRefLabel.ParentFont := True;
    DetRefLabel.Caption := '(not found)';
  end
  else begin
    //DetRefLabel.Color := $00FF00;
    //DetRefLabel.Font.Color := $000000;
    DetRefLabel.Caption := 'FOUND @ ' + IntToStr(RefDI);
  end;

  if TestDI = diNone then begin
    //DetTestLabel.ParentColor := True;
    //DetTestLabel.ParentFont := True;
    DetTestLabel.Caption := '(none found)';
  end
  else if TestDI = diOnly then begin
    //DetTestLabel.ParentColor := True;
    //DetTestLabel.ParentFont := True;
    DetTestLabel.Caption := '(too many found)';
  end
  else begin
    //DetTestLabel.Color := $00FF00;
    //DetTestLabel.Font.Color := $000000;
    DetTestLabel.Caption := 'FOUND @ ' + IntToStr(TestDI);
  end;



  if TestDI = diOnly then TestDI := diNone;



  bGotEm := (RefDI <> diNone) and (TestDI <> diNone);
  TestOldButton.Enabled := bGotEm;
  TestNewButton.Enabled := bGotEm;

  if RefDI = diNone then begin
    if TestDI = diNone then
      ThisDetState := dsNone
    else
      ThisDetState := dsTest
    ;
  end
  else begin
    if TestDI = diNone then
      ThisDetState := dsRef
    else
      ThisDetState := dsBoth
    ;
  end;
  if ThisDetState <> LastDetState then begin
    LastDetState := ThisDetState;

    SetMasterPassState(psBlank);
  end;
end;

function CalcDIORaw(I: Integer): LongWord;
begin
  if I < $100 then
    Result := $01010101 * I
  else if I < $120 then
    Result := not ($FFFFFFFF shl (I - $120))
  else if I < $140 then
    Result := $FFFFFFFF shl (I - $120)
  else //Not supposed to happen, just don't fuck up if it does.
    Result := I
  ;
end;

function LogLine(Lin: String): Integer;
begin
  Result := MainForm.ResultMemo.Lines.Add(Lin);
end;

procedure PostLine(Lin: String);
var
  I: Integer;

begin
  WaitForSingleObject(hPostedStrMutex, INFINITE);
    I := Length(PostedStr);
    SetLength(PostedStr, I+1);
    PostedStr[I] := Lin;
  ReleaseMutex(hPostedStrMutex);
  PostMessage(MainForm.Handle, AM_POSTSTR, 0, 0);
end;

procedure TMainForm.AMPostStr(var Msg: TMessage);
var
  I: Integer;

begin
  WaitForSingleObject(hPostedStrMutex, INFINITE);
    for I := 0 to High(PostedStr) do
      LogLine(PostedStr[I])
    ;
    SetLength(PostedStr, 0);
  ReleaseMutex(hPostedStrMutex);
end;

procedure DoReport(ReportLine: Integer; bFailed: Boolean);
var
  ReportStr: String;

begin
  if bFailed then
    ReportStr := 'FAIL'
  else
    ReportStr := 'PASS'
  ;
  with MainForm.ResultMemo do Lines[ReportLine] := Lines[ReportLine] + ReportStr;
end;

function BoardName(DI: LongWord): String;
begin
  if DI = RefDI then
    Result := 'REFERENCE board'
  else
    Result := 'board under test'
  ;
end;

procedure NoteDisconnect(DI: LongWord);
begin
  if DI = RefDI then
    bDiscoRef := True
  else
    bDiscoTest := True
  ;
end;

function TryDIOConfigFailed(DI: LongWord; OutMask: Byte; DIOData: LongWord; TriMask: Byte): Boolean;
var
  Status: LongWord;
  ActionStr: String;

begin
  Status := DIO_ConfigureEx(DI, @OutMask, @DIOData, @TriMask);
  Result := Status <> ERROR_SUCCESS;
  if Result then begin
    if TriMask <> 0 then
      ActionStr := ' tristating ' + BoardName(DI) + '.'
    else
      ActionStr := ' configuring ' + BoardName(DI) + '(' + IntToHex(OutMask, 1) + ').'
    ;
    LogLine('Error ' + IntToStr(Status) + ActionStr);

    if Status = ERROR_DEVICE_REMOVED then NoteDisconnect(DI);
  end;
end;

function TryDIOWriteAllFailed(DI: LongWord; DIOData: LongWord): Boolean;
var
  Status: LongWord;

begin
  Status := DIO_WriteAll(DI, @DIOData);
  Result := Status <> ERROR_SUCCESS;
  if Result then begin
    LogLine('Error ' + IntToStr(Status) + ' writing ' + BoardName(DI) + ' DIO.');

    if Status = ERROR_DEVICE_REMOVED then NoteDisconnect(DI);
  end;
end;

function TryDIOReadAllFailed(DI: LongWord; out DIOData: LongWord): Boolean;
var
  Status: LongWord;

begin
  DIOData := 0;
  Status := DIO_ReadAll(DI, @DIOData);
  Result := Status <> ERROR_SUCCESS;
  if Result then begin
    LogLine('Error ' + IntToStr(Status) + ' reading ' + BoardName(DI) + ' DIO.');

    if Status = ERROR_DEVICE_REMOVED then NoteDisconnect(DI);
  end;
end;

function TryDIOStreamOpenFailed(DI: LongWord; bIsRead: Boolean): Boolean;
var
  Status: LongWord;

begin
  Status := DIO_StreamOpen(DI, bIsRead);
  Result := Status <> ERROR_SUCCESS;
  if Result then begin
    LogLine('Error ' + IntToStr(Status) + ' opening stream on ' + BoardName(DI) + '.');

    if Status = ERROR_DEVICE_REMOVED then NoteDisconnect(DI);
  end;
end;

function TryClearFIFO(DI: LongWord): Boolean;
var
  Status: LongWord;
begin
  Status := AIOUSB_ClearFIFO(DI, 0);
  Result := Status <> ERROR_SUCCESS;
  if Result then begin
    LogLine('Error ' + IntToStr(Status) + ' clearing FIFO on ' + BoardName(DI) + '.');

    if Status = ERROR_DEVICE_REMOVED then NoteDisconnect(DI);
  end;
end;

{
procedure _ReportDIOConfig(DI: LongWord);
var
  Status: LongWord;
  OutMask, TriMask: Byte;

begin
  OutMask := $FF;
  TriMask := $FF;
  Status := DIO_ConfigurationQuery(DI, @OutMask, @TriMask);
  if Status <> ERROR_SUCCESS then begin
    LogLine('Error ' + IntToStr(Status) + ' querying config on ' + BoardName(DI) + '.');
    Exit;
  end;
  LogLine(BoardName(DI) + ' has outs=' + IntToHex(OutMask, 1) + ' and tristates=' + IntToHex(TriMask, 1) + '.');
end;
}

function ExpectedFIFOBytes(DI: LongWord): Integer;
var
  PID: LongWord;

begin
  QueryDeviceInfo(DI, @PID, nil, nil, nil, nil);
  if PID = $800C then
    Result := 128*1024
  else
    Result := 8*1024
  ;
end;

procedure CheckFIFOSizes(ReadDI, WriteDI: LongWord);
const
  EPBuffers = 8;
  OtherEPBufferBytes = 512 * (EPBuffers - 1);

var
  LastFIFOPackets, TotFIFOBytes, RefFIFOBytes, TestFIFOBytes, ExpectedTestFIFOBytes: Integer;
  CheckFIFOWriteThread: TCheckFIFOWriteThread;
  BytesTransferred: LongWord;
  InBuf: array of Word;
  Hz,slaveHz:double;
begin
  Hz:=2000000;
  slaveHz := 0;
  DIO_StreamSetClocks(WriteDI, Hz, Hz);
  DIO_StreamSetClocks(ReadDI, slaveHz, slaveHz);

    LogLine('Checking FIFO sizes...');
  //Tristate while setting direction, so it's safe for the control lines.
  TryDIOConfigFailed( RefDI, $8, $FFFFFFFF, $FF);
  TryDIOConfigFailed(TestDI, $4, $FFFFFFFF, $FF);
  DIO_StreamOpen(ReadDI, True);
  DIO_StreamClose(ReadDI);
  DIO_StreamOpen(WriteDI, False);
  TryDIOConfigFailed( RefDI, $8, $FFFFFFFF, $0);
  TryDIOConfigFailed(TestDI, $4, $FFFFFFFF, $0);
  //Launch thread to write, with no read, until it locks up.
  CheckFIFOWriteThread := TCheckFIFOWriteThread.Create(True);
  CheckFIFOWriteThread.DI := WriteDI;
  CheckFIFOWriteThread.Resume;
  //Wait for it to lock up.
  LastFIFOPackets := 0;
  repeat
    Sleep(1); //Should be more than enough for the thread to transfer one packet.
    if CheckFIFOWriteThread.FIFOPackets = LastFIFOPackets then Break;

    LastFIFOPackets := CheckFIFOWriteThread.FIFOPackets;
  until False;
  TotFIFOBytes := 512 * CheckFIFOWriteThread.FIFOPackets;
  //Tell the thread to not write any more, and then read out data so it can release.
  CheckFIFOWriteThread.Terminate;
  DIO_StreamOpen(ReadDI, True);
  SetLength(InBuf, 256);
  DIO_StreamFrame(ReadDI, Length(InBuf), @InBuf[0], BytesTransferred);
  LastFIFOPackets := 1;
  Sleep(1); //Make sure the thread has time to report its last packet.
  repeat
    DIO_StreamFrame(ReadDI, Length(InBuf), @InBuf[0], BytesTransferred);
    Inc(LastFIFOPackets);
  until LastFIFOPackets = CheckFIFOWriteThread.FIFOPackets;
  DIO_StreamClose(ReadDI);
  DIO_StreamClose(WriteDI);
  //Clean up worker thread.
  FreeAndNil(CheckFIFOWriteThread);

  //Calc and report.
  RefFIFOBytes := ExpectedFIFOBytes(RefDI);
  TestFIFOBytes := TotFIFOBytes - RefFIFOBytes - OtherEPBufferBytes;
  ExpectedTestFIFOBytes := ExpectedFIFOBytes(TestDI);

  if TestFIFOBytes = ExpectedTestFIFOBytes then begin
    LogLine('FIFO sizes match.');
    Exit;
  end;

  LogLine('Warning:');
  LogLine('FIFO size mismatch detected!');
  LogLine('Reference board FIFO: ' + IntToStr(RefFIFOBytes div 1024) + 'kB');
  LogLine('Bytes assumed held in other EP buffers: ' + FloatToStr(OtherEPBufferBytes / 1024) + 'kB');
  LogLine('Board under test FIFO: ' + FloatToStr(TestFIFOBytes / 1024) + 'kB measured, expected ' + IntToStr(ExpectedTestFIFOBytes div 1024) + 'kB from PID.');
  LogLine('Board under test may need to be re-KeyMastered!');
end;

function StrForMHz(MHz: LongWord): String;
begin
  if MHz = 0 then
    Result := ''
  else
    Result := ' (' + IntToStr(MHz) + 'MHz clock source)'
  ;
end;

function DoTest(bOldBoard: Boolean): TPassState;
var
  Status: LongWord;
  StuckHigh, StuckLow,
  DIORaw, SlowDIOIn, SlowDIOOut: LongWord;
  bFailed, bThisFailed, bOptimistic: Boolean;
  iTest, iWord, iDevice,
  DoneCode, Mismatches: Integer;
  ReadHz, WriteHz: Double;
  WriteThread: TWriteThread;
  ReadThread: TReadThread;
  Naui, StartTime, ElapsedTime, TestDur: Int64;
  ReportLine, ThisReportLine: Integer;
  DescStr: String;
  bFastSafeIsDI: Boolean;
  RefMHz, TestMHz: String;

  procedure NoWorky;
  begin
    LogLine('This isn''t going to work.');
    DoReport(ReportLine, True);
    Result := psFail;
  end;

  procedure ItDisconnected;
  begin
    LogLine('');
    LogLine('DISCONNECTION:');
    if bDiscoRef then LogLine('Reference board disconnected.');
    if bDiscoTest then LogLine('Board under test disconnected.');
    LogLine('Aborting test.');
    Result := psFail;
  end;

  procedure CheckAndReportSlowMismatch(DIData, DOData: LongWord);
  begin
    if DIData <> DOData then begin
      bFailed := True;

      if Mismatches = 0 then
        LogLine('Mismatch: ' + IntToHex(DIData, 8) + ' XOR ' + IntToHex(DOData, 8) + ' = ' + IntToHex(DIData xor DOData, 8))
      else if Mismatches = 1 then
        LogLine('More mismatches...')
      ;
      Inc(Mismatches);

      StuckHigh := StuckHigh or (DIData and not DOData);
      StuckLow  := StuckLow  or (DOData and not DIData);
    end;
  end;

  procedure ReportStucks;
  var
    MoreComplexErrors: LongWord;
    
  begin
    MoreComplexErrors := StuckHigh and StuckLow;
    StuckHigh := StuckHigh and not MoreComplexErrors;
    StuckLow  := StuckLow  and not MoreComplexErrors;
    if StuckHigh <> 0 then LogLine('Bits that may be stuck high: ' + IntToHex(StuckHigh, 8));
    if StuckLow  <> 0 then LogLine('Bits that may be stuck low: '  + IntToHex(StuckLow , 8));
    if MoreComplexErrors <> 0 then PostLine('Bits with more complex errors: '  + IntToHex(MoreComplexErrors, 4));
    StuckHigh := 0;
    StuckLow  := 0;
  end;

begin
  Result := psTesting;

  bDiscoRef := False;
  bDiscoTest := False;

  ReportLine := LogLine('Basic Config: ');

  bFailed := False;
  if TryDIOConfigFailed(RefDI, $8, $FFFFFFFF, $FF) then begin
    NoWorky;
    Exit;
  end;
  if TryDIOConfigFailed(TestDI, $4, $FFFFFFFF, $FF) then begin
    NoWorky;
    Exit;
  end;
  if bFailed then Result := psFail;

  //Put the fast ports into opposite directions, so it's safe for the control lines.
  //Set it for writing from ref to test, unless the board under test is a DO, then do the opposite.
  bFastSafeIsDI := bTestingDI;
  if TryDIOStreamOpenFailed(RefDI, not bFastSafeIsDI) then begin
    NoWorky;
    Exit;
  end;
  Status := DIO_StreamClose(RefDI);
  if Status <> ERROR_SUCCESS then begin
    LogLine('Error ' + IntToStr(Status) + ' closing stream on REFERENCE board.');
    NoWorky;
    Exit;
  end;
  if TryDIOStreamOpenFailed(TestDI, bFastSafeIsDI) then begin
    NoWorky;
    Exit;
  end;
  Status := DIO_StreamClose(TestDI);
  if Status <> ERROR_SUCCESS then begin
    LogLine('Error ' + IntToStr(Status) + ' closing stream on board under test.');
    NoWorky;
    Exit;
  end;
  DoReport(ReportLine, False);
  LogLine('');





  LogLine('Slow DIO');

  ReportLine := LogLine('Write ABC, Read D: ');
  bFailed := False;
  if TryDIOConfigFailed(TestDI, $7, $FFFFFFFF, $0) then bFailed := True;
  if TryDIOConfigFailed( RefDI, $8, $FFFFFFFF, $0) then bFailed := True;
  if bFailed then Result := psFail;

  Mismatches := 0;
  StuckHigh := 0;
  StuckLow := 0;
  for iTest := 0 to $13F do begin
    DIORaw := CalcDIORaw(iTest);
    if TryDIOWriteAllFailed(TestDI, DIORaw    ) then bFailed := True;
    if TryDIOWriteAllFailed(RefDI , DIORaw    ) then bFailed := True;
    if  TryDIOReadAllFailed(TestDI, SlowDIOOut) then bFailed := True;
    if  TryDIOReadAllFailed(RefDI , SlowDIOIn ) then bFailed := True;
    CheckAndReportSlowMismatch(
      (SlowDIOIn  and $00FFFFFF) or (SlowDIOOut and $FF000000)
    ,
      (SlowDIOOut and $00FFFFFF) or (SlowDIOIn  and $FF000000)
    );

    if bDiscoRef or bDiscoTest then begin
      ItDisconnected;
      DoReport(ReportLine, True);
      Exit;
    end;
  end;

  if TryDIOConfigFailed( RefDI, $4, $FFFFFFFF, $FF) then bFailed := True;
  if TryDIOConfigFailed(TestDI, $8, $FFFFFFFF, $FF) then bFailed := True;
  DoReport(ReportLine, bFailed);
  if bFailed then Result := psFail;
  ReportStucks;



  ReportLine := LogLine('Write D, Read ABC: ');
  bFailed := False;
  if TryDIOConfigFailed(TestDI, $8, $FFFFFFFF, $0) then bFailed := True;
  if TryDIOConfigFailed( RefDI, $7, $FFFFFFFF, $0) then bFailed := True;
  if bFailed then Result := psFail;

  Mismatches := 0;
  StuckHigh := 0;
  StuckLow := 0;
  for iTest := 0 to $13F do begin
    DIORaw := CalcDIORaw(iTest);
    if TryDIOWriteAllFailed(RefDI , DIORaw    ) then bFailed := True;
    if TryDIOWriteAllFailed(TestDI, DIORaw    ) then bFailed := True;
    if  TryDIOReadAllFailed(RefDI , SlowDIOOut) then bFailed := True;
    if  TryDIOReadAllFailed(TestDI, SlowDIOIn ) then bFailed := True;
    CheckAndReportSlowMismatch(
      (SlowDIOIn  and $00FFFFFF) or (SlowDIOOut and $FF000000)
    ,
      (SlowDIOOut and $00FFFFFF) or (SlowDIOIn  and $FF000000)
    );

    if bDiscoRef or bDiscoTest then begin
      ItDisconnected;
      DoReport(ReportLine, True);
      Exit;
    end;
  end;

  if TryDIOConfigFailed( RefDI, $4, $FFFFFFFF, $FF) then bFailed := True;
  if TryDIOConfigFailed(TestDI, $8, $FFFFFFFF, $FF) then bFailed := True;
  DoReport(ReportLine, bFailed);
  if bFailed then Result := psFail;
  ReportStucks;





  if Result = psFail then begin
    LogLine('Skipping fast DIO because of previous failures.');
    Exit;
  end;

  LogLine('');
  ReportLine := LogLine('Fast DIO: ');
  {//Looks like clearing the FIFO makes the boards go crazy.
  bFailed := False;
  if not TryClearFIFO(RefDI) then bFailed := True;
  if not TryClearFIFO(TestDI) then bFailed := True;
  if bFailed then Result := psFail;
  }

  bFailed := False;
  for iTest := 0 to High(TestScript) do with TestScript[iTest] do begin

    if WriteDI = RefDI then begin
      if not bTestingDI then Continue;
      DescStr := 'Reference board' + StrForMHz(WriteMasterMHz) + ' writing to test board' + StrForMHz(ReadMasterMHz) + '.';
    end
    else begin
      if not bTestingDO then Continue;
      DescStr := 'Test board' + StrForMHz(WriteMasterMHz) + ' writing to reference board' + StrForMHz(ReadMasterMHz) + '.';
    end;
    //DescStr := DescStr + FloatToStr(Max(ReadMasterMHz, WriteMasterMHz)) + ' MHz, ';
    //if ReadMasterMHz <> 0 then DescStr := DescStr + 'In master: ' else DescStr := DescStr + 'Out master: ';
    ThisReportLine := LogLine(DescStr);
    bThisFailed := False;
    bOptimistic := True;

    if TryDIOStreamOpenFailed(WriteDI, False) then bThisFailed := True;
    if TryDIOStreamOpenFailed(ReadDI, True) then bThisFailed := True;

    ReadHz := 0;
    WriteHz := 1024*1024*WriteMasterMHz;
    Status := DIO_StreamSetClocks(WriteDI, ReadHz, WriteHz);
    if Status <> ERROR_SUCCESS then begin
      LogLine('Error ' + IntToStr(Status) + ' setting clocks on ' + BoardName(WriteDI) + '.');
      bThisFailed := True;
    end;
    ReadHz := 1024*1024*ReadMasterMHz;
    WriteHz := 0;
    Status := DIO_StreamSetClocks(ReadDI, ReadHz, WriteHz);
    if Status <> ERROR_SUCCESS then begin
      LogLine('Error ' + IntToStr(Status) + ' setting clocks on ' + BoardName(ReadDI) + '.');
      bThisFailed := True;
    end;

    if TryDIOConfigFailed( RefDI, $4, $FFFFFFFF, $1) then bThisFailed := True;
    if TryDIOConfigFailed(TestDI, $8, $FFFFFFFF, $1) then bThisFailed := True;

    OutBuf[0] := (WriteDI shl 4) or ReadDI;
    OutBuf[1] := (OutBuf[0] shl 4) or 1;
    for iWord := 2 to WriteWords - 1 do
      OutBuf[iWord] := OutBuf[1] * iWord
    ;
    WriteThread := TWriteThread.Create;
    WriteThread.DI := WriteDI;
    ReadThread := TReadThread.Create;
    ReadThread.DI := ReadDI;
    ReadThread.bEatFirstWord := bOldBoard;// and (iTest = 0);
    ReadThread.Resume;
    Sleep(1);
    WriteThread.Resume;
    QueryPerformanceCounter(StartTime);
    TestDur := 32*TicksPerSec;
    repeat
      Sleep(1);
      Application.ProcessMessages;

      QueryPerformanceCounter(Naui);
      ElapsedTime := Naui - StartTime;
      if bOptimistic and (ElapsedTime > 4*TicksPerSec) then begin
        LogLine(#9'If this doesn''t lock up, the test may need to be re-run...');
        bOptimistic := False;
      end;
    until (WriteThread.IsDone and ReadThread.IsDone) or (ElapsedTime > TestDur);

    if TryDIOConfigFailed( RefDI, $4, $FFFFFFFF, $FF) then bThisFailed := True;
    if TryDIOConfigFailed(TestDI, $8, $FFFFFFFF, $FF) then bThisFailed := True;

    Application.ProcessMessages;

    DoneCode := 0;
    if WriteThread.IsDone then Inc(DoneCode, 1);
    if ReadThread.IsDone then Inc(DoneCode, 2);
    if DoneCode = 3 then begin
      if WriteThread.bFailed or ReadThread.bFailed then bThisFailed := True;
      WriteThread.Free;
      ReadThread.Free;
    end
    else begin
      case DoneCode of
        0: LogLine('Both DIO threads locked up.');
        1: begin LogLine('DIO read thread locked up on ' + BoardName(ReadDI) + '.'); ReadThread.Terminate; WriteThread.Free; end;
        2: begin LogLine('DIO write thread locked up on ' + BoardName(WriteDI) + '.'); WriteThread.Terminate; ReadThread.Free; end;
      end;
      LogLine('');
      LogLine('PLEASE UNPLUG. The test can''t continue.');
      LogLine('');
      LogLine('If this happens too many times, you''ll also need to close and rerun this test, or it''ll crash/lock up/etc.');
      DoReport(ThisReportLine, True);
      DoReport(ReportLine, True);
      Result := psFail;
      for iDevice := 0 to 31 do AIOUSB_CloseDevice(iDevice);
      Exit;
    end;

    DoReport(ThisReportLine, bThisFailed);
    if bThisFailed then bFailed := True;
  end;
  DoReport(ReportLine, bFailed);
  if bFailed then Result := psFail;





  if Result = psTesting then Result := psPass;
end;

procedure TMainForm.TestButtonClick(Sender: TObject);
var
  OverallResult: TPassState;
  bOldBoard: Boolean;

  ReportLine,
  iTest: Integer;

begin
  bOldBoard := Sender = TestOldButton;

  ResultMemo.Lines.Clear;

  if RefDI = diNone then begin
    ResultMemo.Lines.Add('No reference board found.');
    SetMasterPassState(psBlank);
    Exit;
  end;
  if TestDI = diNone then begin
    ResultMemo.Lines.Add('No board under test found.');
    SetMasterPassState(psBlank);
    Exit;
  end;

  SetMasterPassState(psTesting);
  CheckTime.Enabled := False;
  Application.ProcessMessages;

  SetLength(OutBuf, WriteWords);
  SetLength(InBuf, 4*ReadWords);

  for iTest := 0 to High(TestScript) do
    if (iTest and 2) = 0 then begin
      TestScript[iTest].ReadDI := RefDI;
      TestScript[iTest].WriteDI := TestDI;
    end
    else begin
      TestScript[iTest].ReadDI := TestDI;
      TestScript[iTest].WriteDI := RefDI;
    end
  ;

  ReportLine := LogLine('Overall: ');
  LogLine('');

  OverallResult := DoTest(bOldBoard);
  LogLine('');

  if OverallResult = psPass then begin
    if bTestingDO then
      CheckFIFOSizes(RefDI, TestDI)
    else
      CheckFIFOSizes(TestDI, RefDI)
    ;
  end;

  if TryDIOConfigFailed( RefDI, $8, $FFFFFFFF, $FF) then OverallResult := psFail;
  if TryDIOConfigFailed(TestDI, $4, $FFFFFFFF, $FF) then OverallResult := psFail;

  SetLength(OutBuf, 0);
  SetLength(InBuf, 0);

  CheckTime.Enabled := True;

  if OverallResult = psPass then begin
    SetMasterPassState(psPass);
    DoReport(ReportLine, False);
  end
  else begin
    SetMasterPassState(psFail);
    DoReport(ReportLine, True);
  end;
  LogLine('');
end;

{ TWriteThread }

procedure TWriteThread.Execute;
var
  Status, BytesTransferred: LongWord;

begin
  Status := DIO_StreamFrame(DI, WriteWords, @OutBuf[0], BytesTransferred);
  if Status <> ERROR_SUCCESS then begin
    PostLine('Error ' + IntToStr(Status) + ' writing frame to ' + BoardName(DI) + '.');
    bFailed := True;
  end;
  DIO_StreamClose(DI);
  bDone := True;
end;

{ TReadThread }

procedure TReadThread.Execute;
var
  Status, FramePointsLeft, BytesTransferred,
  StuckHigh, StuckLow, MoreComplexErrors: LongWord;
  Mismatches,
  iReadFrame, iWord, iOWord: Integer;

begin
  Mismatches := 0;
  StuckHigh := 0;
  StuckLow := 0;
  for iReadFrame := 0 to 3 do begin
    if Terminated then Break;

    FramePointsLeft := ReadWords;
    repeat
      Status := DIO_StreamFrame(DI, FramePointsLeft, @InBuf[iReadFrame*ReadWords + ReadWords - Integer(FramePointsLeft)], BytesTransferred);
      if Terminated then Break;
      if Status <> ERROR_SUCCESS then begin
        bFailed := True;

        PostLine('Error ' + IntToStr(Status) + ' reading frame from ' + BoardName(DI) + '.');
        Break;
      end;
      if BytesTransferred = 2*FramePointsLeft then Break;

      Dec(FramePointsLeft, BytesTransferred div 2);
    until False;
    if Terminated then Break;
    if Status <> ERROR_SUCCESS then Continue;

    for iWord := iReadFrame*ReadWords to iReadFrame*ReadWords + ReadWords - 1 do begin
      iOWord := iWord - Ord(bEatFirstWord);
      if iOWord < 0 then Continue;

      if InBuf[iWord] <> OutBuf[iOWord] then begin
        bFailed := True;

        StuckHigh := StuckHigh or (InBuf[iWord] and not OutBuf[iOWord]);
        StuckLow  := StuckLow  or (OutBuf[iOWord] and not InBuf[iWord]);

        if Terminated then Break;

        if Mismatches < 8 then
          PostLine(#9'Mismatch @ ' + IntToHex(iOWord, 1) + ': ' + IntToHex(InBuf[iWord], 4) + ' <> ' + IntToHex(OutBuf[iOWord], 4))
        else if Mismatches = 8 then
          PostLine(#9'More mismatches...')
        ;
        Inc(Mismatches);
      end;
    end;
  end;

  DIO_StreamClose(DI);
  if not Terminated then begin
    MoreComplexErrors := StuckHigh and StuckLow;
    StuckHigh := StuckHigh and not MoreComplexErrors;
    StuckLow  := StuckLow  and not MoreComplexErrors;
    if StuckHigh <> 0 then PostLine(#9'Bits that may be stuck high: ' + IntToHex(StuckHigh, 4));
    if StuckLow  <> 0 then PostLine(#9'Bits that may be stuck low: '  + IntToHex(StuckLow , 4));
    if MoreComplexErrors <> 0 then PostLine(#9'Bits with more complex errors: '  + IntToHex(MoreComplexErrors, 4));
  end;
  bDone := True;
end;

{ TIOThread }

constructor TIOThread.Create;
begin
  inherited Create(True);

  //hDoneMutex := CreateMutex(nil, False, nil);
end;

destructor TIOThread.Destroy;
begin
  //CloseHandle(hDoneMutex);

  inherited;
end;

function TIOThread.IsDone: Boolean;
begin
  if not bDone then WaitForSingleObject(Handle, 1);

  Result := bDone;
end;

{ TCheckFIFOWriteThread }

procedure TCheckFIFOWriteThread.Execute;
var
  Status, BytesTransferred: LongWord;
  OutBuf: array of Word;

begin
  bFailed := False;
  FIFOPackets := 0;
  SetLength(OutBuf, 256);
  FillChar(OutBuf[0], Length(OutBuf)*SizeOf(OutBuf[0]), 0);

  repeat
    Status := DIO_StreamFrame(DI, Length(OutBuf), @OutBuf[0], BytesTransferred);
    if Status <> ERROR_SUCCESS then begin
      PostLine('Error ' + IntToStr(Status) + ' writing frame to ' + BoardName(DI) + '.');
      bFailed := True;
      Break;
    end;
    Inc(FIFOPackets);
  until Terminated;
end;


end.
